@@ -1,8 +1,8 @@
-import socket, re
+import socket, re, sys
 from codecs import encode, decode
 from . import shared
 
-def get_whois_raw(domain, server="", previous=[], rfc3490=True, never_cut=False):
+def get_whois_raw(domain, server="", previous=[], rfc3490=True, never_cut=False, with_server_list=False, server_list=[]):
 	# Sometimes IANA simply won't give us the right root WHOIS server
 	exceptions = {
 		".ac.uk": "whois.ja.net",
@@ -12,12 +12,16 @@ def get_whois_raw(domain, server="", previous=[], rfc3490=True, never_cut=False)
 	}
 	
 	if rfc3490:
-		domain = encode( domain if type(domain) is unicode else decode(domain, "utf8"), "idna" )
+		if sys.version_info < (3, 0):
+			domain = encode( domain if type(domain) is unicode else decode(domain, "utf8"), "idna" )
+		else:
+			domain = encode(domain, "idna").decode("ascii")
 
-	if len(previous) == 0:
+	if len(previous) == 0 and server == "":
 		# Root query
+		server_list = [] # Otherwise it retains the list on subsequent queries, for some reason.
 		is_exception = False
-		for exception, exc_serv in exceptions.iteritems():
+		for exception, exc_serv in exceptions.items():
 			if domain.endswith(exception):
 				is_exception = True
 				target_server = exc_serv
@@ -26,7 +30,7 @@ def get_whois_raw(domain, server="", previous=[], rfc3490=True, never_cut=False)
 			target_server = get_root_server(domain)
 	else:
 		target_server = server
-	if domain.endswith(".jp") and target_server == "whois.jprs.jp":
+	if target_server == "whois.jprs.jp":
 		request_domain = "%s/e" % domain # Suppress Japanese output
 	elif domain.endswith(".de") and ( target_server == "whois.denic.de" or target_server == "de.whois-servers.net" ):
 		request_domain = "-T dn,ace %s" % domain # regional specific stuff
@@ -52,14 +56,18 @@ def get_whois_raw(domain, server="", previous=[], rfc3490=True, never_cut=False)
 				break
 	if never_cut == False:
 		new_list = [response] + previous
+	server_list.append(target_server)
 	for line in [x.strip() for x in response.splitlines()]:
 		match = re.match("(refer|whois server|referral url|whois server|registrar whois):\s*([^\s]+\.[^\s]+)", line, re.IGNORECASE)
 		if match is not None:
 			referal_server = match.group(2)
-			if referal_server != server:
+			if referal_server != server and "://" not in referal_server: # We want to ignore anything non-WHOIS (eg. HTTP) for now.
 				# Referal to another WHOIS server...
-				return get_whois_raw(domain, referal_server, new_list)
-	return new_list
+				return get_whois_raw(domain, referal_server, new_list, server_list=server_list, with_server_list=with_server_list)
+	if with_server_list:
+		return (new_list, server_list)
+	else:
+		return new_list
 	
 def get_root_server(domain):
 	data = whois_request(domain, "whois.iana.org")
@@ -73,11 +81,11 @@ def get_root_server(domain):
 def whois_request(domain, server, port=43):
 	sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
 	sock.connect((server, port))
-	sock.send("%s\r\n" % domain)
-	buff = ""
+	sock.send(("%s\r\n" % domain).encode("utf-8"))
+	buff = b""
 	while True:
 		data = sock.recv(1024)
 		if len(data) == 0:
 			break
 		buff += data
-	return buff
+	return buff.decode("utf-8")
